////////////////////////////////////////////////////////
//    
//
//	 PIC Thermal Data Logger   2010-07-04 
//
//			2010-07-04 Ver1.0  Fujitec-Labs
//    Use CCS-C PCM Compiler 
////////////////////////////////////////////////////////

#include <16lf1827.h>
//#include <string.h>

#fuses INTRC_IO, NOWDT, NOLVP, NOBROWNOUT, NOPROTECT, NOIESO, NOFCMEN
#device ADC=10
#use delay(clock=500000)				// System Clk 500KHz
#use RS232(BAUD=2400, XMIT=PIN_B2, RCV=PIN_B1, PARITY=N, BITS=8, STOP=1)

// System Port Assignment
// 
// RA0 = SDO		 FRAM Serial Data out
// RA1 = CS		 	FRAM CS
// RA2 = FRAM_PCON  0: ON  1: OFF
// RA3 = VCONT 		1: Vcc= 3.3V  0: Vcc=2.1V
// RA4 = RAM_DET_IN  0: FRAM_DETECT   1: FRAM NOT DETECT 
// RA5 = RESET_IN
// RA6 = SCK		FRAM SCK
// RA7 = SDI_IN		FRAM Serial Data in
// RB0 = DC_IN(INT)  0: DC_IN	1: NO DC_IN
// RB1 = RXD
// RB2 = TXD
// RB3 = SBAT_MON (ANALOG)		Battery Voltage Check
// RB4 = SENSOR_IN (ANALOG)		Thermal Sensor MCP9700 
// RB5 = SENSOR_VCC 1: ON		MCP9700 VCC Supply
// RB6 = T1OSCIN				32.768KHZ XTAL
// RB7 = T1OSCOUT				32.769KHz XTAL	

//  2.1V Mode ɂRB2 ̃|[gݒ胂[hςTXD ̃xL ɂ邱ƁBiłȂƓdĂ܂j
// PICC ̃foCX`t@C@PIC16LF1827.h  Interrupt Global `C邱
// #define GLOBAL    @ 0x0B80@----> 0x0BC0


/// Proto Type 
void init_calendar();
void inc_calendar();
void save_calendar();
void read_sensor();
void cal_average_data();
long save_fram(long address);
void  wr_streg(int data);
void wren();
void wrdi();
void read_status_reg();
void write_status_reg();
void write_fram(long address, int data);
int read_fram(long address);
int write_byte(int data);
void batt_check();

/// O[oϐA萔錾
#BYTE T1CON 	= 0x18 
#BYTE OSCCON 	= 0x99
#BYTE OSCSTAT 	= 0x9A

#BYTE PORTA 	= 0x0c
#BYTE PORTB 	= 0x0d
#BYTE TRISA 	= 0x8C
#BYTE TRISB 	= 0x8D
#BYTE T1CON 	= 0x18
#BYTE T1GCON 	= 0x19
#BYTE TMR1L 	= 0x16
#BYTE TMR1H 	= 0x17
#BYTE PIE1 		= 0x91
#BYTE PIR1 		= 0x11
#BYTE PIR2 		= 0x12
#BYTE PIR3 		= 0x13
#BYTE PIR4 		= 0x14

#BYTE INTCON 	= 0x0b
#BYTE PIE1 		= 0x91
#BYTE TXREG 	= 0x19A
#WORD RCSTA 	= 0x19D
#BYTE ADCON0 	= 0x9D
#BYTE ADCON1 	= 0x9E
#BYTE FVRCON 	= 0x117

#BIT T1OSCEN 	= T1CON.3
#BIT TMR1GE 	= T1GCON.7
#BIT GIE 		= INTCON.7 
#BIT T1OSCR		= OSCSTAT.7
#BIT TMR1IF 	= PIR1.0
#BIT TMR1IE 	= PIE1.0
#BIT SPEN 		= RCSTA.7
#BIT TXD 		= PORTB.2
#BIT DCIN 		= PORTB.0
#BIT  SDO 		= PORTA.0
#BIT  CS 		= PORTA.1
#BIT  FRAM_PCON = PORTA.2
#BIT  VCONT		= PORTA.3
#BIT  RAMDET 	= PORTA.4
#BIT  SCK 		= PORTA.6
#BIT  SDI 		= PORTA.7
#BIT SENSOR_VCC = PORTB.5
#BIT SENSOR_IN 	= PORTB.4
#BIT ADON 		= ADCON0.0
#BIT FVREN 		= FVRCON.7
#BIT FVRRDY 	= FVRCON.6
#BIT ADFVR1 	= FVRCON.1
#BIT ADFVR0 	= FVRCON.0
#BIT ADNREF 	= ADCON1.2
#BIT ADPREF1 	= ADCON1.1
#BIT ADPREF0 	= ADCON1.0


int  data, flag, counter;
int  tm_int_flag, c;
int  year, month, day, hour, min, sec;
int  sec10_flag, min_flag;
int  th_data[6];
int  vcc_data[6];
int data_pointer, av_th_data, av_vcc_data;
int thrm_data, bat_data, lowbat_flag, DCIN_flag;

unsigned long fram_address;
unsigned long adc_data;

// Timer Interrupt Routine
#int_timer1				 
void timet_1_exec()
	{
	TMR1H=0x80;			// 1 sec interval
	TMR1L=0x00;	
	counter ++;	
	tm_int_flag = 1;	// Set Timer Interrupt Flag
	enable_interrupts(INT_TIMER1);
	}	

/// C֐
void main()
	{
	setup_oscillator(OSC_500KHz);
	setup_adc_ports(sAN8);
	setup_adc_ports(sAN9);	// Sensor IN, BAT IN, Internal VREF 2.048V	// Sensor IN, BAT IN, Internal VREF 2.048V
	ADFVR1 = 1;				// ADC reference Voltage is 2.048V
	ADFVR0 = 0;
	ADNREF = 0;				// AD Negative reference is GND
	ADPREF1 = 1;			// AD Positive reference is internal 2.048V
	ADPREF0 = 1;
	setup_adc(ADC_CLOCK_DIV_2);	// OSC=1MHz DIV=2 łADϊ12clk ̂= 48uSec
	set_tris_a (0xb0);
    set_tris_b (0xdb);

	VCONT = 0;				// Set VCC 1.8V
//	VCONT = 1;				// Set VCC 3.3V	For Debug

// FRAM Port Initialize
	CS = 0;
	SCK = 0;
	SDO = 0;

// Power Initialize
	SENSOR_VCC = 0;
	FRAM_PCON = 1;
	
// 32.768KHz OSC Initialize
    T1OSCEN = 1;			// External 32.768KHz Xtal Enable 
	TMR1GE = 0;				// TMR1GE OFF 
	T1CON = 0b10001101;		// Timer-1 EXT_OSC, DIV_BY_1, T1OSCEN, TMR1ON 32K XTAL 
//	T1CON = 0b00111001;		// Timer-1 INT OSC, DIV_BY_8, T1OSCEN, TMR1ON 
	TMR1IF = 0;				// CLR TM1 INT FLAG
	TMR1H=0x80;			// CLR TM1 Register
	TMR1L=0x00;	

	enable_interrupts(INT_TIMER1);
 	enable_interrupts(GLOBAL);
	
	if (DCIN == 0)				// If External DC is sullpied for Debug
		{
		SPEN = 1;				// UART ENABLE	
		VCONT = 1;				// VCC is 3.3V
		delay_ms(100);			// Wait for voltage stable
		printf("\r\n");			// Send open message
		printf ("START\r\n");
		}
	else						// Normal Operation
		{
		SPEN = 0;				// UART DISABLE		
		TXD= 0;					// SET TXD OUTPUT = L
		VCONT = 0;				// VCC 2.1V	
		}
	counter = 0;			// Ticked data for debug
	tm_int_flag = 0;		// Timer Interrupt Flag clear
	sec10_flag = 0; 		// 10 sec flag
	min_flag = 0;			// Min flag 
	fram_address = 0x100;	// FRAM Data Store start address 
	init_calendar();		// Set saved Calendar data in FRAM, otherwise set default data 

	while(1)				// Main Loop
		{	
		if(DCIN == 0)
			{
			DCIN_flag = 1;	// If External DC_IN is supplied, set DCIN_flag
			}
		else
			{
			DCIN_flag = 0;	// Clear DCIN_flag in Battery Mode
			}

		if (DCIN_flag == 1)		// EXT DCIN
			{
			VCONT = 1;			// Set VCC 3.3V
			delay_ms(1);		// Delay for voltage stability
			SPEN = 1;			// UART ENABLE
			}

		if (tm_int_flag == 1)		// 1Sec Interrupt process
			{
			inc_calendar();			// Calendar increment
			if (sec10_flag == 1)	// 10 sec ?
				{
				VCONT = 1;			// VCC 3.3V
				delay_ms(1);		// Delay for voltage stability
				read_sensor();		// Read Battery Voltage and Thermal Sensor
			    batt_check();		// Battery voltage check  0.9V=70, 1.0V =75, 1.2V=90			
				sec10_flag = 0;
				}						
			if ((min_flag ==1) && (lowbat_flag == 0))	// 1 Min ?
				{
				cal_average_data();		// Calculate average data
				VCONT = 1;				// VCC 3.3V
				delay_ms(1);			// Delay for voltage stability
				fram_address = save_fram(fram_address);		// Save calendar and average data to FRAM
				min_flag = 0;
				}
			if (DCIN_flag == 1)	// EXT DC IN ??  Debug Monitor 
				{
				read_sensor();
				printf ("20%d, %d, %d, %d, %d, %d, %u, %u, %d\r\n", year, month, day, hour, min, sec, thrm_data, bat_data, lowbat_flag);
				}
			tm_int_flag = 0;			// Clear Timer Interrupt Flag
			}
		if(DCIN_flag == 0)			// BATT MODE ?
			{			
			CS = 0;					// FRAM CS = L
			SCK = 0;				// FRAM SCK = L
			SDO = 0;				// FRAM SDO = L
			FRAM_PCON = 1;			// FRAM VCC OFF
			SENSOR_VCC = 0;			// Thermo Sensor OFF
			SPEN = 0;				// UART DISABLE
			TXD= 0;					// SET TXD OUTPUT = L
			ADFVR1 = 0;				// ADC reference output is OFF
			ADFVR0 = 0;				// 
			FVREN = 0;				// FVR OFF		
			VCONT = 0;				// Set VCC 1.8V
			sleep();				// SLEEP (Available only in EXT 32KHz mode)		
			}	
		}
	}


// Read Initial Calendar Data from FRAM and set data
// If fram_data[0][1] is "CA" then read data and set Calendar, otherwise set as 2010-Jan-01-00-00.
void init_calendar()				
	{
	long address;
	int fram_data [8];
	int i;

	address = 0x0000;
	if(RAMDET == 0)		// FRAM connection check
		{
		FRAM_PCON = 0;	// FRAM VCC ON
		CS = 1;			// FRAM PORT initialize
		delay_ms(1);	// delay
		SCK = 1;
		SDO = 1;
		for (i= 0; i<8 ; i++)			// Read Initial data from FRAM
			{
			fram_data[i] = read_fram(address + i);
			}
		SCK = 0;		// Port = L
		SDO = 0;		// Port = L
		delay_ms(1);	// delay	
		CS = 0;			// FRAM PORT reset
		FRAM_PCON = 1;	// FRAM VCC OFF
		}
	if (RAMDET == 0 && fram_data[0] == 'C' && fram_data[1] == 'A')
		{
		year = fram_data[2];
		month = fram_data[3];
		day = fram_data[4];
		hour = fram_data[5];
	 	min = fram_data[6];
	 	sec = fram_data[7];
		}
	else
		{	
		year = 10;
		month = 1;
		day = 1;
		hour = 0;
		min = 0;
		sec = 0;
		}
	}

// Calendar Increment 
// Not care for Leap Year
void inc_calendar()
	{
	sec ++;
	if (sec == 10 || sec == 20 || sec == 30 || sec == 40 || sec == 50 || sec == 60)
		{
		sec10_flag = 1;		// 10 sec flag 
		}
	if (sec == 60)
		{
		sec = 0;
		min ++;				// Increment Min
		min_flag = 1;
		if (min == 60)
			{
			min = 0;
			hour ++;			// Increment Hour
			if (hour == 24)
				{
				hour = 0;
				day ++;
				if (day == 28 && month == 2)	// February is special
					{
					day = 1;
					month ++;
					}
				if (day == 31 && (month == 4 || month == 6 || month == 9 || month == 11))
					{
					day = 1;
					month ++;
					}
				if (day == 32 && (month == 1 || month == 3 || month == 5 || month == 7
					|| month == 8 || month == 10 || month ==12))
					{
					day = 1;
					month ++;
					}	
				if (month == 13)
					{
					month = 1;
					year ++;		// Increment Year
					}
				}
			}
		}
	}

// Save Calendar data to FRAM
void save_calendar()
	{
	long address;
	int fram_data [8];
	int i;

	address = 0x0000;	// FRAM Start address	
	if(RAMDET == 0)		// FRAM connection check
		{
		delay_ms(1);	// delay
		FRAM_PCON = 0;	// FRAM VCC ON
		CS = 1;			// FRAM PORT initialize
		delay_ms(1);	// delay
		SCK = 1;
		SDO = 1;
		fram_data[0] = 'C';		// Header "CA"
		fram_data[1] = 'A';		// 
		fram_data[2] = year;
		fram_data[3] = month;
		fram_data[4] = day;
		fram_data[5] = hour;
		fram_data[6] = min;
		fram_data[7] = sec;
		wr_streg(0);	// Write status register as unprotect
		for (i= 0; i<8 ; i++)
			{
			write_fram(address + i, fram_data[i]);	// Store data to FRAM
			}
		SCK = 0;		// Port = L
		SDO = 0;		// Port = L
		delay_ms(1);	// delay	
		CS = 0;			// FRAM PORT reset
		FRAM_PCON = 1;	// FRAM VCC OFF
		}
	}


// Read MCP9700 Thermal Sensor and Battery Voltage  
void read_sensor()
	{
	SENSOR_VCC = 1;				// Thermo Sensor VCC ON
	ADON = 1;					// ADC Enable
	ADFVR1 = 1;					// ADC reference output is 2.048V
	ADFVR0 = 0;					// 
	FVREN = 1;					// FVR ON
	while(FVRRDY != 1)			// FVR is Ready ?
		{
		}	
	set_adc_channel(8);			// Sensor Select
	delay_us(1000);				// Wait for voltage stable
	adc_data = read_adc();		// Thermo Sensor Voltage Check
	thrm_data = (adc_data >> 2) & 0x00ff;	// High 8 bit
	th_data[data_pointer] = thrm_data;	// save data
	
	set_adc_channel(9);			// Solar BATT Select
	delay_us(1000);
	adc_data = read_adc();		// Solar Bat Voltage Check
	bat_data = (adc_data >> 2) & 0x00ff;	// save data
	vcc_data[data_pointer] = bat_data;	// High 8 bit
	ADON = 0;					// ADC OFF
	ADFVR1 = 0;					// ADC reference output is OFF
	ADFVR0 = 0;					// 
	FVREN = 0;					// FVR OFF	
	SENSOR_VCC =0;				// Thermo Sensor VCC OFF
	data_pointer ++;			// pointer increment
	if (data_pointer == 6)		
		{
		data_pointer = 0;
		}
	}	

// Check Battery Voltage 
void batt_check()
	{
	if (bat_data <= 112)		// Batt is under 0.9V
		{
		lowbat_flag = 1;
		}
	else
		{
		lowbat_flag = 0;
		}		
	}

//// Calculate average value from 6 point data
void cal_average_data()
	{
	int i;
	unsigned long sum_th_data;
	unsigned long sum_vcc_data;

	sum_th_data = 0x0000;
	sum_vcc_data = 0x0000;
	for (i=0; i< 6; i++)
		{
		sum_th_data = sum_th_data + th_data[i];
		sum_vcc_data = sum_vcc_data + vcc_data[i];
		}
	av_th_data = sum_th_data /6;
	av_vcc_data = sum_vcc_data /6;
	}		

// Calendar Data Save to FRAM
long save_fram(long address)
	{
	int i;
	int fram_data[8];

	if(RAMDET == 0)		// FRAM connection check
		{
		FRAM_PCON = 0;	// FRAM VCC ON
		CS = 1;			// FRAM PORT initialize
		delay_ms(1);	// delay
		SCK = 1;
		SDO = 1;
		fram_data[0] = ':';
		fram_data[1] = year;
		fram_data[2] = month;
		fram_data[3] = day;
		fram_data[4] = hour;
		fram_data[5] = min;
		fram_data[6] = av_th_data;
		fram_data[7] = av_vcc_data;
		wr_streg(0);	// Write status register unprotect
		for (i= 0; i<8 ; i++)
			{
			write_fram(address + i, fram_data[i]);
			}
		address = address + 8;
		if (address == 0x8000)
			{
			address= 0x0100;
			}
		SCK = 0;		// Port = L
		SDO = 0;		// Port = L
		delay_ms(1);	// delay	
		CS = 0;			// FRAM PORT reset
		FRAM_PCON = 1;	// FRAM VCC OFF
		return (address);
		}
	}							
								


//// Write FRAM status register (Low Level Function)
void  wr_streg(int data)
{
	wren();				// WEL set
	CS = 0;
	write_byte(0x01);
	write_byte(data);	//
	CS = 1;
	delay_us(100);
}

//// Read FRAM Status Register (Low Level Function)
int rd_streg()
{
	int data;

	CS = 0;
	write_byte(0x05);
	data = write_byte(0x00);	// dummy
	CS = 1;
	delay_us(100);
	return(data);
}

//// Set FRAM WEL iWrite Enable Latch) 
void wren()
{
	CS = 0;
	write_byte(0x06);	// WREN
	CS = 1;
	delay_us(100);
}

//// Reset FRAM WEL iWrite Enable Latchj
void wrdi()
{
	CS = 0;
	write_byte(0x04);	// WRDI
	CS = 1;
	delay_us(100);
}


//// Data Write FRAM 
void write_fram(long address, int data)
{
	wren();				// WEL set
	CS = 0;
	write_byte(0x02);
	write_byte(address >> 8) & 0x7F;
 	write_byte(address);
 	write_byte(data);
  	CS = 1;
	delay_us(100);
}

//// Data Read FRAM
int read_fram(long address)
{
	int data;

	CS = 0;
	write_byte(0x03);
	write_byte(address >> 8) & 0x7F;
 	write_byte(address);
	data = write_byte(0x00); // dummy
  	CS = 1;
	delay_us(100);
	return(data);
}

//// Data Write FRAM (Low Level Function)
int write_byte(int data)
{
	int rdata, i;

	rdata = 0x00;
	for (i=0; i<8; i++)
		{
		if ((data & 0x80) == 0x80)
			SDO = 1;
		else
			SDO = 0;
		data = (data << 1);
		SCK = 0;
		delay_us(100);
		SCK = 1;
		rdata = (rdata << 1);
		if(SDI == 1)
			rdata = (rdata | 0x01);
		else
			rdata = (rdata & 0xfe);
		}
	return (rdata);
}



