////////////////////////////////////////////////////
//
//	 FRAM TEST TOOL  
//			Ver1.0    2010-07-07
//			Fujitec-Labs
//    Use CCS-C PCM Compiler
////////////////////////////////////////////////////			


#include <16f88.h>

#fuses INTRC_IO, NOWDT, NOPUT,  NOLVP, NOBROWNOUT, NOPROTECT
#use delay(clock=8000000)		// Use internal 8MHz OSC
#use fast_io(A)
#use fast_io(B)
#use RS232(BAUD=9600, XMIT=PIN_B5, RCV=PIN_B2, PARITY=N, BITS=8, STOP=1)

// System Port Assignment
//
// RA0 = SDO (O)		FRAM Serial Data Out
// RA1 = CS (O)			FRAM CS
// RA2 = 
// RA3 = 
// RA4 =  
// RA5 = RESET_IN (I)
// RA6 = SCK (O)		FRAM SCK
// RA7 = SDI_IN (I)		FRAM Serial Data In
// RB0 = RAMDET (I)		FRAM_DETECT
// RB1 = 
// RB2 = RXD			
// RB3 = 
// RB4 = 
// RB5 = TXD
// RB6 = 
// RB7 = 


/// Proto Type 
long datacnv();
void verify_mem();
void test_mem();
void clear_mem();
void fill_mem();
void disp_mem();
void load_mem();
void help();
void wren();
void wrdi();
void wrsr(int data);
int rdsr();
void wr_streg(int data);
int rd_streg();
void read_status_reg();
void write_status_reg();
void write_fram(long address, int data);
int read_fram(long address);
int write_byte(int data);

static char keybuf[10];			// receive buffer
static char tempbuf[10];

/// O[oϐA萔錾

#define MAX_FRAM 0x8000		// Maximum 0x8000

#BYTE PORTA 		= 0x05
#BYTE PORTB 		= 0x06
#BYTE OPTION_REG 	= 0x81

#BIT  SDO 			= PORTA.0
#BIT  CS 			= PORTA.1
#BIT  SCK 			= PORTA.6
#BIT  SDI 			= PORTA.7
#BIT  RAMDET 		= PORTB.0
#BIT RBPU 			= OPTION_REG.7



	
/// C֐
void main()
{
	setup_oscillator(OSC_8MHz);
	set_tris_a (0xa0);
    set_tris_b (0b11000101);
	printf ("START\r\n");
	
// FRAM Port Initialize
	CS = 1;
	SDO = 1;
	SCK = 1;

	while(1)			// Main Loop
	{
		keybuf[0]=0;
		printf("\r\nCommand= ");
		gets(keybuf);

		// Command Analyze
		switch(keybuf[0])			
		{
			case 'c': clear_mem(); break;	// Memory Clear	
			case 'f': fill_mem(); break;	// Fill one block
			case 'd': disp_mem(); break;	// Read one block
			case 's': test_mem(); break;	// Write one Byte
			case 'v': verify_mem(); break;	// Read one Byte
			case 'w': write_status_reg(); break;  // Write Status Register
			case 'r': read_status_reg(); break;   // Read Status Register
			case 'l': load_mem(); break;		// FRAM Calendar data (Formatted)
			case 'h': help(); break;			// Help menu
			default: printf("\r\nError?");	//@Command Error
		}
	}
}

void test_mem()
{
	long addr;
	int  data;
	
	printf("\r\nDeviceAddress= ");
	gets(keybuf);
	addr = datacnv();

	printf("\r\ndata= ");
	gets(keybuf);
	data = datacnv();

	printf("\r\n DeviceAddress= %04LX,  data= %02x\r\n", addr, data);

	write_fram(addr, data);		// Write Data
	data=read_fram(addr);	
	printf("\r\nr_data= %x\r\n", data);
}

void verify_mem()
{
	long addr;
	int data;
	
	printf("\r\nDeviceAddress= ");
	gets(keybuf);
	addr = datacnv();

	data=read_fram(addr);	
	printf("\r\n DeviceAddress= %04LX, r_data= %02x\r\n", addr, data);
}
	


long datacnv()
{
	int i;
	long tempdata;

	for(i=0; i<10; i++)				// temp buffer clear
		tempbuf[i]=0;		
	for(i=0; keybuf[i] != 0x00;i++)
	{
		if((keybuf[i] >= '0') && (keybuf[i] <= '9'))
			{
			keybuf[i] = keybuf[i] & 0x0f;
			}
		else if (((keybuf[i] >= 'A') && (keybuf[i] <='F')) || ((keybuf[i] >= 'a') && (keybuf[i] <='f')))
			{
			keybuf[i] = (keybuf[i] & 0x0f) + 0x09;
			}
		else
			keybuf[i]=0x00;

		tempbuf[3]=tempbuf[2];
		tempbuf[2]=tempbuf[1];
		tempbuf[1]=tempbuf[0];
		tempbuf[0]=keybuf[i];
	}
	tempdata = tempbuf[3]*4096 + tempbuf[2]*256 + tempbuf[1]*16 + tempbuf[0];
	return (tempdata);
}


//// Memory Clear
void clear_mem()				// write all 0x00
{
	long address;
	long start_addr;
	long last_addr;

	printf("\r\nStart Address= ");
	gets(keybuf);
	start_addr = datacnv();
	
	printf("\r\nLast Address = ");
	gets(keybuf);
	last_addr = datacnv();

	printf("\r\nClearing ");

	if (last_addr >= MAX_FRAM)
		last_addr = MAX_FRAM;
	for (address=start_addr; address< last_addr; address++)
	{
		if((address > 16) && ((address % 16) == 0))
			printf(".");
		write_fram(address, 0);
	}
	printf("\r\nComplete!\r\n");
}

/// Fill Memory
void fill_mem()
{
	int data;
	long address;
	long start_addr;
	long last_addr;

	printf("\r\nStart Address= ");
	gets(keybuf);
	start_addr = datacnv();

	printf("\r\nLast Address = ");
	gets(keybuf);
	last_addr = datacnv();

	printf("\r\ndata= ");
	gets(keybuf);
	data = datacnv();

	if (last_addr >= MAX_FRAM)
		last_addr = MAX_FRAM;
	printf("\r\n");
	for (address=start_addr; address< last_addr; address++)
	{
		if((address > 16) && ((address % 16) == 0))
			printf(".");
		write_fram(address, data);
	}
printf("\r\nComplete!\r\n");					
}

//// Dump Memory
void disp_mem()
{
	int data, j;
	long address;
	long start_addr;
	long last_addr;

	printf("\r\nStart Address= ");
	gets(keybuf);
	start_addr = datacnv();

	printf("\r\nLast Address = ");
	gets(keybuf);
	last_addr = datacnv();

//	last_addr = addr+256;
	if (last_addr >= MAX_FRAM)
		last_addr = MAX_FRAM;
	
    address = start_addr;
	do
	{
		printf("\r\n%04LX  ", address);			// Address
		for(j=0; j<16; j++)						// Disp 16 byte
		{
			data=read_fram(address);
			address++;
			printf("%x ", data);
		}
	} while(address < last_addr);				// Check last address
	printf("\r\nComplete!\r\n");
}

//// Convert FRAM Data with Decimal Format
void load_mem()
{
	int data, j;
	long address;
	long addr;
	long last_address;

	printf("\r\nStart Address= ");
	gets(keybuf);
	addr = datacnv();
	printf("\r\nLast Address= ");
	gets(keybuf);
	last_address = datacnv();

	if (last_address >= MAX_FRAM)
		last_address = MAX_FRAM;
	
    address = addr;
	do
	{
		printf("\r\n%04LX  ", address);			//Disp Address
		for(j=0; j<8; j++)						// Disp 8 byte
		{
			data=read_fram(address);
			address++;
			printf("%u ", data);				// Disp Data with unsigned Decimal 
		}
	} while(address < last_address);				// Check last address
	printf("\r\nComplete!\r\n");
}

//// Write FRAM Status Register
void  write_status_reg()
{
	int  data;
	
	printf("\r\ndata= ");
	gets(keybuf);
	data = datacnv();
	wr_streg(data);

	data= rd_streg();	
	printf("\r\n status_reg= %x\r\n", data);
}

//// Read FRAM Status Register
void read_status_reg()
{
	int data;
	
	data=rd_streg();	
	printf("\r\n status_reg= %02x\r\n", data);
}

//// Write FRAM Status Register (Low Level function)
void  wr_streg(int data)
{
	wren();				// WEL set
	CS = 0;
	write_byte(0x01);
	write_byte(data);	//
	CS = 1;
	delay_us(100);
}

//// Read FRAM Status Register (Low Level Function)
int rd_streg()
{
	int data;

	CS = 0;
	write_byte(0x05);
	data = write_byte(0x00);	// dummy
	CS = 1;
	delay_us(100);
	return(data);
}

//// Set FRAM WEL (Write Enable Latch)
void wren()
{
	CS = 0;
	write_byte(0x06);	// WREN
	CS = 1;
	delay_us(100);
}

//// Reset FRAM WEL (Write Enable Latch)
void wrdi()
{
	CS = 0;
	write_byte(0x04);	// WRDI
	CS = 1;
	delay_us(100);
}


//// Write FRAM 
void write_fram(long address, int data)
{
	wren();				// WEL set
	CS = 0;
	write_byte(0x02);
	write_byte(address >> 8) & 0x7F;
 	write_byte(address);
 	write_byte(data);
  	CS = 1;
	delay_us(100);
}

//// Read FRAM 
int read_fram(long address)
{
	int data;

	CS = 0;
	write_byte(0x03);
	write_byte(address >> 8) & 0x7F;
 	write_byte(address);
	data = write_byte(0x00); // dummy
  	CS = 1;
	delay_us(100);
	return(data);
}

//// Write FRAM (Low Level Function)
int write_byte(int data)
{
	int rdata, i;

	rdata = 0x00;
	for (i=0; i<8; i++)
		{
		if ((data & 0x80) == 0x80)
			SDO = 1;
		else
			SDO = 0;
		data = (data << 1);			// Shift Left
		SCK = 0;
		delay_us(100);
		SCK = 1;
		rdata = (rdata << 1);
		if(SDI == 1)
			rdata = (rdata | 0x01);		//Data is '1'
		else
			rdata = (rdata & 0xfe);		// Data is '0'
		}
	return (rdata);
}

void help()
{
	printf("\r\n");
	printf("  'c':  NA\r\n");	
	printf("  'f':  ubN\r\n");
	printf("  'd':  ubNǂݏo\r\n");
	printf("  's':  PoCg\r\n");
	printf("  'v':  PoCgmF\r\n");
	printf("  'w':  Xe[^XWX^\r\n");
	printf("  'r':  Xe[^XWX^mF\r\n");
	printf("  'l':  FRAM Dump\r\n");
}



